@@ -100,11 +100,14 @@ class Filter(object):
 
     ### Utility methods needed by most/all subclasses ###
 
-    def param_from_choices(self, choices):
+    def param_from_choice(self, choice):
+        return unicode(choice)
+
+    def paramlist_from_choices(self, choices):
         """
         For a list of choices, return the parameter list that should be created.
         """
-        return map(unicode, choices)
+        return map(self.param_from_choice, choices)
 
     def build_params(self, add=None, remove=None):
         """
@@ -121,7 +124,7 @@ class Filter(object):
             if add not in chosen:
                 chosen.append(add)
         if chosen:
-            params.setlist(self.query_param, self.param_from_choices(chosen))
+            params.setlist(self.query_param, self.paramlist_from_choices(chosen))
         else:
             del params[self.query_param]
         params.pop('page', None) # links should reset paging
@@ -144,14 +147,23 @@ class Filter(object):
         chosen = self.chosen
         choices = []
         for choice in chosen:
-            display = self.display_choice(choice)
-            if display is not None:
-                choices.append(FilterChoice(display,
-                                            None, # Don't need count for removing
-                                            self.build_params(remove=[choice]),
-                                            FILTER_REMOVE))
+            choices.append(FilterChoice(self.render_choice_object(choice),
+                                        None, # Don't need count for removing
+                                        self.build_params(remove=[choice]),
+                                        FILTER_REMOVE))
         return choices
 
+    def render_choice_object(self, choice_obj):
+        """
+        Converts an object that is available for choosing (that usually is the
+        result of a database lookup) or has been chosen already into a unicode
+        object for display.
+
+        The choice object could be the 'raw' query string or database value,
+        or transformed into something more convenient (e.g. a model instance)
+        """
+        return unicode(choice_obj)
+
 
 class SingleValueMixin(object):
     """
@@ -229,7 +241,7 @@ class SimpleQueryMixin(object):
         return value_counts(qs, self.field)
 
 
-class RangeFilterMixin(ChooseAgainMixin, SingleValueMixin):
+class RangeFilterMixin(ChooseAgainMixin):
 
     # choice_type must be set to a class that provides the static method
     # 'from_param' and instance methods 'make_lookup' and 'display', and the
@@ -247,9 +259,6 @@ class RangeFilterMixin(ChooseAgainMixin, SingleValueMixin):
     def lookup_from_choice(self, choice):
         return choice.make_lookup(self.field)
 
-    def display_choice(self, choice):
-        return choice.display()
-
     def get_choices_remove(self, qs):
         # Due to drill down, if a broader param is removed, the more specific
         # params must be removed too. We assume we can do an ordering on
@@ -259,7 +268,7 @@ class RangeFilterMixin(ChooseAgainMixin, SingleValueMixin):
         out = []
         for i, choice in enumerate(chosen):
             to_remove = [c for c in chosen if c >= choice]
-            out.append(FilterChoice(self.display_choice(choice),
+            out.append(FilterChoice(self.render_choice_object(choice),
                                     None,
                                     self.build_params(remove=to_remove),
                                     FILTER_REMOVE))
@@ -272,8 +281,8 @@ class ValuesFilter(ChooseOnceMixin, SimpleQueryMixin, Filter):
     """
     Fallback Filter for various kinds of simple values.
     """
-    def display_choice(self, choice):
-        retval = unicode(choice)
+    def render_choice_object(self, choice):
+        retval = super(ValuesFilter, self).render_choice_object(choice)
         if retval == u'':
             return u'(empty)'
         else:
@@ -284,7 +293,7 @@ class ValuesFilter(ChooseOnceMixin, SimpleQueryMixin, Filter):
         Called by 'get_choices', this is usually the one to override.
         """
         count_dict = self.get_values_counts(qs)
-        return [FilterChoice(self.display_choice(val),
+        return [FilterChoice(self.render_choice_object(val),
                              count,
                              self.build_params(add=val),
                              FILTER_ADD)
@@ -303,7 +312,7 @@ class ChoicesFilter(ValuesFilter):
         super(ChoicesFilter, self).__init__(*args, **kwargs)
         self.choices_dict = dict(self.field_obj.flatchoices)
 
-    def display_choice(self, choice):
+    def render_choice_object(self, choice):
         # 3) above
         return self.choices_dict.get(choice, choice)
 
@@ -313,9 +322,7 @@ class ChoicesFilter(ValuesFilter):
         for val, display in self.field_obj.choices:
             # 1), 2) above
             if val in count_dict:
-                # We could use the value 'display' here, but for consistency
-                # call display_choice() in case it is overriden.
-                choices.append(FilterChoice(self.display_choice(val),
+                choices.append(FilterChoice(self.render_choice_object(val),
                                             count_dict[val],
                                             self.build_params(add=val),
                                             FILTER_ADD))
@@ -326,14 +333,17 @@ class ForeignKeyFilter(ChooseOnceMixin, SimpleQueryMixin, RelatedObjectMixin, Fi
     """
     Filter for ForeignKey fields.
     """
-    def display_choice(self, choice):
-
-        lookup = {self.rel_field.name: choice}
+    def choice_from_param(self, param):
+        choice_pk = super(ForeignKeyFilter, self).choice_from_param(param)
+        lookup = {self.rel_field.name: choice_pk}
         try:
             obj = self.rel_model.objects.get(**lookup)
         except self.rel_model.DoesNotExist:
-            return None
-        return unicode(obj)
+            raise ValueError("object does not exist in DB")
+        return obj
+
+    def param_from_choice(self, choice):
+        return unicode(choice.pk)
 
     def get_choices_add(self, qs):
         count_dict = self.get_values_counts(qs)
@@ -343,9 +353,9 @@ class ForeignKeyFilter(ChooseOnceMixin, SimpleQueryMixin, RelatedObjectMixin, Fi
 
         for o in objs:
             pk = getattr(o, self.rel_field.attname)
-            choices.append(FilterChoice(unicode(o),
+            choices.append(FilterChoice(self.render_choice_object(o),
                                         count_dict[pk],
-                                        self.build_params(add=pk),
+                                        self.build_params(add=o),
                                         FILTER_ADD))
         return choices
 
@@ -381,25 +391,30 @@ class ManyToManyFilter(ChooseAgainMixin, RelatedObjectMixin, Filter):
         # Now, need to lookup objects on related table, to display them.
         objs = self.rel_model.objects.filter(pk__in=count_dict.keys())
 
-        return [FilterChoice(unicode(o),
+        return [FilterChoice(self.render_choice_object(o),
                              count_dict[o.pk],
-                             self.build_params(add=o.pk),
+                             self.build_params(add=o),
                              FILTER_ADD)
                 for o in objs]
 
-    def get_choices_remove(self, qs):
-        chosen = self.chosen
-        # Do a query in bulk to get objs corresponding to choices.
-        objs = self.rel_model.objects.filter(pk__in=chosen)
+    def param_from_choice(self, choice):
+        return unicode(choice.pk)
 
-        # We want to preserve order of items in params, so use the original
-        # 'chosen' list, rather than objs.
+    def choices_from_params(self):
+        # To create the model instances, we override this method rather than
+        # choice_from_param in order to do a single bulk query rather than
+        # multiple queries. So 'choice_from_param' technically returns the
+        # wrong type of thing, since it returns PKs not instances.
+        chosen_pks = super(ManyToManyFilter, self).choices_from_params()
+        objs = self.rel_model.objects.filter(pk__in=chosen_pks)
+        # Now need to get original order back. But also need to be aware
+        # that some things may not exist in DB
         obj_dict = dict([(obj.pk, obj) for obj in objs])
-        return [FilterChoice(unicode(obj_dict[choice]),
-                             None, # Don't need count for removing
-                             self.build_params(remove=[choice]),
-                             FILTER_REMOVE)
-                for choice in chosen if choice in obj_dict]
+        retval = []
+        for c in chosen_pks:
+            if c in obj_dict:
+                retval.append(obj_dict[c])
+        return retval
 
 
 class DateRangeType(object):
@@ -488,11 +503,11 @@ class DateChoice(object):
         if self.range_type.single:
             value = self.values[0]
             parts = value.split('-')
-            if self.range_type == YEAR:
+            if self.range_type is YEAR:
                 return parts[0]
-            elif self.range_type == MONTH:
+            elif self.range_type is MONTH:
                 return unicode(MONTHS[int(parts[1])])
-            elif self.range_type == DAY:
+            elif self.range_type is DAY:
                 return str(int(parts[-1]))
         else:
             return u'-'.join([DateChoice(DateRangeType.get(self.range_type.level, True),
@@ -501,9 +516,9 @@ class DateChoice(object):
 
     @staticmethod
     def datetime_to_value(range_type, dt):
-        if range_type == YEAR:
+        if range_type is YEAR:
             return '%04d' % dt.year
-        elif range_type == MONTH:
+        elif range_type is MONTH:
             return '%04d-%02d' % (dt.year, dt.month)
         else:
             return '%04d-%02d-%02d' % (dt.year, dt.month, dt.day)
@@ -566,41 +581,83 @@ class DateTimeFilter(RangeFilterMixin, Filter):
         self.max_depth_level = self.max_depth_levels[self.max_depth]
         super(DateTimeFilter, self).__init__(*args, **kwargs)
 
+    def render_choice_object(self, choice):
+        return choice.display()
+
+    def get_choices_remove(self, qs):
+        chosen = list(self.chosen)
+        out = []
+
+        for i, choice in enumerate(chosen):
+            # As for RangeFilterMixin, if a broader param is removed, the more
+            # specific params must be removed too.
+            to_remove = [c for c in chosen if c >= choice]
+            out.append(FilterChoice(self.render_choice_object(choice),
+                                    None,
+                                    self.build_params(remove=to_remove),
+                                    FILTER_REMOVE))
+            # There can be cases where there are gaps, so we need to bridge
+            # using FILTER_DISPLAY
+            out.extend(self.bridge_choices(chosen[0:i+1], chosen[i+1:]))
+        return out
+
     def get_choices_add(self, qs):
         chosen = list(self.chosen)
-        range_type = None
 
-        if len(chosen) > 0:
-            range_type = chosen[-1].range_type.drilldown()
+        # For the case of needing to drill down past a single option
+        # to get to some real choices, we define a recursive
+        # function.
+
+        def get_choices_add_recursive(chosen):
+            range_type = None
+
+            if len(chosen) > 0:
+                range_type = chosen[-1].range_type.drilldown()
+                if range_type is None:
+                    return []
+
             if range_type is None:
-                return []
-
-        if range_type is None:
-            # Get some initial idea of range
-            date_range = qs.aggregate(first=models.Min(self.field),
-                                      last=models.Max(self.field))
-            first = date_range['first']
-            last = date_range['last']
-            if first is None or last is None:
-                # No values, can't drill down:
-                return []
-            if first.year == last.year:
-                if first.month == last.month:
-                    range_type = DAY
+                # Get some initial idea of range
+                date_range = qs.aggregate(first=models.Min(self.field),
+                                          last=models.Max(self.field))
+                first = date_range['first']
+                last = date_range['last']
+                if first is None or last is None:
+                    # No values, can't drill down:
+                    return []
+                if first.year == last.year:
+                    if first.month == last.month:
+                        range_type = DAY
+                    else:
+                        range_type = MONTH
                 else:
-                    range_type = MONTH
+                    range_type = YEAR
+
+            date_qs = qs.dates(self.field, range_type.label)
+            results = date_aggregation(date_qs)
+
+            date_choice_counts = self.collapse_results(results, range_type)
+            if len(date_choice_counts) == 1 and range_type is not None:
+                # Single choice - recurse.
+                single_choice, count = date_choice_counts[0]
+                date_choice_counts_deeper = get_choices_add_recursive([single_choice])
+                if len(date_choice_counts_deeper) == 0:
+                    # Nothing there, so ignore
+                    return date_choice_counts
+                else:
+                    # We discard date_choice_counts, because bridge_choices will
+                    # make it up again.
+                    return date_choice_counts_deeper
             else:
-                range_type = YEAR
+                return date_choice_counts
 
-        date_qs = qs.dates(self.field, range_type.label)
-        results = date_aggregation(date_qs)
-
-        date_choice_counts = self.collapse_results(results, range_type)
+        date_choice_counts = get_choices_add_recursive(chosen)
 
         choices = []
         # Additional display links, to give context for choices if necessary.
         if len(date_choice_counts) > 0:
-            choices.extend(self.bridge_choices(chosen, date_choice_counts))
+            choices.extend(self.bridge_choices(chosen,
+                                               [choice for choice, count in date_choice_counts]))
 
         for date_choice, count in date_choice_counts:
             if date_choice in chosen:
@@ -608,13 +665,20 @@ class DateTimeFilter(RangeFilterMixin, Filter):
 
             # To ensure we get the bridge choices, which are useful, we check
             # self.max_depth_level late on and bailout here.
-            if range_type.level > self.max_depth_level:
+            if date_choice.range_type.level > self.max_depth_level:
                 continue
 
-            choices.append(FilterChoice(date_choice.display(),
+            if (len(date_choice_counts) == 1 and
+                (date_choice.range_type.level == self.max_depth_level or
+                 count == 1)):
+                link_type = FILTER_DISPLAY
+            else:
+                link_type = FILTER_ADD
+
+            choices.append(FilterChoice(self.render_choice_object(date_choice),
                                         count,
                                         self.build_params(add=date_choice),
-                                        FILTER_ADD))
+                                        link_type))
         return choices
 
     def collapse_results(self, results, range_type):
@@ -622,10 +686,10 @@ class DateTimeFilter(RangeFilterMixin, Filter):
             # If range_type is month/day, we don't want any possibility of the
             # buckets wrapping over to the next year/month, so we set first and
             # last accordingly
-            if range_type == MONTH:
+            if range_type is MONTH:
                 first, last = 1, 12
-            elif range_type == DAY:
-                first, last = 1, 31
+            elif range_type is DAY:
+                first, last = 1, ((results[0][0] + relativedelta(day=1)) + relativedelta(months=1, days=-1)).day
             else:
                 first = results[0][0].year
                 last = results[-1][0].year
@@ -645,12 +709,14 @@ class DateTimeFilter(RangeFilterMixin, Filter):
             date_choice_counts = []
             for i, bucket in enumerate(buckets):
                 count = sum(row[1] for row in bucket)
-                start_val = first + bucketsize * i
-                start_date = dt_template.replace(**dict({range_type.dateattr: start_val}))
-                end_date = start_date + relativedelta(**dict({range_type.relativedeltaattr: bucketsize - 1}))
+                if count:
+                    start_val = first + bucketsize * i
+                    end_val = min(start_val + bucketsize, last)
+                    start_date = dt_template.replace(**dict({range_type.dateattr: start_val}))
+                    end_date = dt_template.replace(**dict({range_type.dateattr: end_val}))
 
-                choice = DateChoice.from_datetime_range(range_type, start_date, end_date)
-                date_choice_counts.append((choice, count))
+                    choice = DateChoice.from_datetime_range(range_type, start_date, end_date)
+                    date_choice_counts.append((choice, count))
         else:
             date_choice_counts = [(DateChoice.from_datetime(range_type, dt), count)
                                   for dt, count in results]
@@ -658,30 +724,41 @@ class DateTimeFilter(RangeFilterMixin, Filter):
 
     def bridge_choices(self, chosen, choices):
         # Returns FILTER_DISPLAY type choices to bridge from what is chosen
-        # (which might be nothing) to the first 'add' link, to give context to
-        # the link.
+        # (which might be nothing) to what can be chosen, to give context to the
+        # link.
+
+        # Note this is used in bridging to the 'add' choices, and in bridging
+        # between 'remove' choices
+
         if len(choices) == 0:
             return []
         if len(chosen) == 0:
             chosen_level = 0
+            bridge_to_single = False
         else:
             chosen_level = chosen[-1].range_type.level
+            bridge_to_single = not chosen[-1].range_type.single
 
         # first choice in list can act as template, as it will have all the
         # values we need.
-        new_choice = choices[0][0]
+        new_choice = choices[0]
         new_level = new_choice.range_type.level
 
         retval = []
-        while chosen_level < new_level - 1:
-            chosen_level += 1
+        while (chosen_level < new_level - 1) or (chosen_level < new_level and bridge_to_single):
+            # If the first chosen was multi, first bridge just bridges to single
+            if bridge_to_single:
+                bridge_to_single = False
+            else:
+                chosen_level += 1
             if chosen_level > self.max_depth_level:
                 continue
             date_choice = DateChoice(DateRangeType.get(chosen_level, True),
                                      new_choice.values)
-            retval.append(FilterChoice(date_choice.display(),
+            retval.append(FilterChoice(self.render_choice_object(date_choice),
                                        None, None,
                                        FILTER_DISPLAY))
+
         return retval
 
 
@@ -759,7 +836,7 @@ def make_numeric_range_choice(to_python, to_str):
     return NumericRangeChoice
 
 
-class NumericRangeFilter(RangeFilterMixin, Filter):
+class NumericRangeFilter(RangeFilterMixin, SingleValueMixin, Filter):
 
     def __init__(self, field, model, params, **kwargs):
         self.max_links = kwargs.pop('max_links', 5)
@@ -769,7 +846,7 @@ class NumericRangeFilter(RangeFilterMixin, Filter):
         self.choice_type = make_numeric_range_choice(field_obj.to_python, str)
         super(NumericRangeFilter, self).__init__(field, model, params, **kwargs)
 
-    def display_choice(self, c):
+    def render_choice_object(self, c):
         if self.ranges is None:
             return c.display()
         if len(c.values) == 1:
@@ -800,7 +877,7 @@ class NumericRangeFilter(RangeFilterMixin, Filter):
             val_counts = value_counts(qs, self.field)
             for v, count in val_counts.items():
                 choice = self.choice_type([RangeEnd(v, True)])
-                choices.append(FilterChoice(self.display_choice(choice),
+                choices.append(FilterChoice(self.render_choice_object(choice),
                                             count,
                                             self.build_params(add=choice),
                                             FILTER_ADD))
@@ -820,7 +897,7 @@ class NumericRangeFilter(RangeFilterMixin, Filter):
                 # the first will include 10 and 20, the second will exlude 20.
                 choice = self.choice_type([RangeEnd(vals[0], i == 0),
                                            RangeEnd(vals[1], True)])
-                choices.append(FilterChoice(self.display_choice(choice),
+                choices.append(FilterChoice(self.render_choice_object(choice),
                                             count,
                                             self.build_params(add=choice),
                                             FILTER_ADD))
