@@ -3,6 +3,7 @@
 from datetime import datetime, date
 from decimal import Decimal
 import operator
+import re
 
 from django.http import QueryDict
 from django.test import TestCase
@@ -13,7 +14,7 @@ from django_easyfilters.filters import \
     FILTER_ADD, FILTER_REMOVE, FILTER_DISPLAY, \
     ForeignKeyFilter, ValuesFilter, ChoicesFilter, ManyToManyFilter, DateTimeFilter, NumericRangeFilter
 
-from models import Book, Genre, Author, BINDING_CHOICES
+from models import Book, Genre, Author, BINDING_CHOICES, Person
 
 
 class TestFilterSet(TestCase):
@@ -66,6 +67,7 @@ class TestFilterSet(TestCase):
                 'authors',
                 'date_published',
                 'price',
+                'rating',
                 ]
 
         fs = BookFilterSet(Book.objects.all(), QueryDict(''))
@@ -75,6 +77,7 @@ class TestFilterSet(TestCase):
         self.assertEqual(ManyToManyFilter, type(fs.filters[3]))
         self.assertEqual(DateTimeFilter, type(fs.filters[4]))
         self.assertEqual(NumericRangeFilter, type(fs.filters[5]))
+        self.assertEqual(NumericRangeFilter, type(fs.filters[6]))
 
     def test_specify_custom_filter(self):
         class AuthorFilterSet(FilterSet):
@@ -362,8 +365,13 @@ class TestFilters(TestCase):
         # If we select 'emily' as an author:
 
         data =  MultiValueDict({'authors':[str(emily.pk)]})
-        filter1 = ManyToManyFilter('authors', Book, data)
-        qs_emily = filter1.apply_filter(qs)
+        with self.assertNumQueries(1):
+            # 1 query for all chosen objects
+            filter1 = ManyToManyFilter('authors', Book, data)
+
+        with self.assertNumQueries(0):
+            # This shouldn't need to do any more queries
+            qs_emily = filter1.apply_filter(qs)
 
         # ...we should get a qs that includes Poems and Wuthering Heights.
         self.assertTrue(qs_emily.filter(name='Poems').exists())
@@ -371,8 +379,13 @@ class TestFilters(TestCase):
         # ...and excludes Jane Eyre
         self.assertFalse(qs_emily.filter(name='Jane Eyre').exists())
 
-        # We should get a 'choices' that includes charlotte and anne
-        choices = filter1.get_choices(qs_emily)
+        with self.assertNumQueries(2):
+            # 0 query for all chosen objects (already done)
+            # 1 query for available objects
+            # 1 query for counts
+            choices = filter1.get_choices(qs_emily)
+
+        # We should have a 'choices' that includes charlotte and anne
         self.assertTrue(unicode(anne) in [c.label for c in choices if c.link_type is FILTER_ADD])
         self.assertTrue(unicode(charlotte) in [c.label for c in choices if c.link_type is FILTER_ADD])
 
@@ -381,9 +394,13 @@ class TestFilters(TestCase):
         # emily should be in 'remove' links, however.
         self.assertTrue(unicode(emily) in [c.label for c in choices if c.link_type is FILTER_REMOVE])
 
-        # If we select again:
-        data2 =  MultiValueDict({'authors': [str(emily.pk), str(anne.pk)]})
-        filter2 = ManyToManyFilter('authors', Book, data2)
+        # Select again - should have sensible params
+        anne_choice = [c for c in choices if c.label.startswith('Anne')][0]
+        self.assertTrue(unicode(emily.pk) in anne_choice.params.getlist('authors'))
+        self.assertTrue(unicode(anne.pk) in anne_choice.params.getlist('authors'))
+
+        # Now do the second select:
+        filter2 = ManyToManyFilter('authors', Book, anne_choice.params)
 
         qs_emily_anne = filter2.apply_filter(qs)
 
@@ -392,10 +409,10 @@ class TestFilters(TestCase):
         # ... but not Wuthering Heights
         self.assertFalse(qs_emily_anne.filter(name='Wuthering Heights').exists())
 
-        # The choices should contain just emily and anne to remove, and
-        # charlotte should have 'link_type' FILTER_ADD. Even though it
+        # The choices should contain just Emily and Anne to remove, and
+        # Charlotte should have 'link_type' FILTER_ADD. Even though it
         # is the only choice, adding the choice is not necessarily the same as
-        # not adding it (could have books by Rmily and Anne, but not charlotte)
+        # not adding it (could have books by Emily and Anne, but not Charlotte)
         choices = filter2.get_choices(qs_emily_anne)
         self.assertEqual([(c.label, c.link_type) for c in choices],
                          [(unicode(emily), FILTER_REMOVE),
@@ -582,7 +599,7 @@ class TestFilters(TestCase):
         # the month should be displayed in 'display' mode.
         qs = Book.objects.filter(id=1)
         params = MultiValueDict(dict(date_published=[str(qs[0].date_published.year)]))
-        f = DateTimeFilter('date_published', Book, params, max_links=10)
+        f = DateTimeFilter('date_published', Book, params, max_links=10, max_depth='month')
 
         choices = f.get_choices(qs)
         self.assertEqual(len(choices), 2)
@@ -683,6 +700,105 @@ class TestFilters(TestCase):
                               ])
 
 
+    def test_datetime_filter_drill_down_to_choice(self):
+        """
+        Tests that if there is a choice that can be displayed, it will drill
+        down to reach it.
+        """
+        # Two birthdays in Jan 2011
+        Person.objects.create(name="Joe", date_of_birth=date(2011, 1, 10))
+        Person.objects.create(name="Peter", date_of_birth=date(2011, 1, 20))
+
+        # Chosen year = 2011
+        params = MultiValueDict({'date_of_birth':['2011']})
+
+        f = DateTimeFilter('date_of_birth', Person, params)
+        qs = Person.objects.all()
+        qs_filtered = f.apply_filter(qs)
+        choices = f.get_choices(qs_filtered)
+
+        # Expect 2011 as remove link
+        self.assertEqual(['2011'], [c.label for c in choices if c.link_type == FILTER_REMOVE])
+        # Expect January as display
+        self.assertEqual(['January'], [c.label for c in choices if c.link_type == FILTER_DISPLAY])
+        # Expect '10' and '20' as choices
+        self.assertEqual(['10', '20'], [c.label for c in choices if c.link_type == FILTER_ADD])
+
+    def test_datetime_filter_remove_choices_complete(self):
+        """
+        Tests that in the case produced in test_datetime_filter_drill_down_to_choice,
+        the remove links display correctly.
+        """
+        # Two birthdays in Jan 2011
+        Person.objects.create(name="Joe", date_of_birth=date(2011, 1, 10))
+        Person.objects.create(name="Peter", date_of_birth=date(2011, 1, 20))
+
+        # Chosen year = 2011, and date = 2011-01-10
+        params = MultiValueDict({'date_of_birth':['2011', '2011-01-10']})
+
+        f = DateTimeFilter('date_of_birth', Person, params)
+        qs = Person.objects.all()
+        qs_filtered = f.apply_filter(qs)
+        choices = f.get_choices(qs_filtered)
+
+        self.assertEqual([('2011', FILTER_REMOVE),
+                          ('January', FILTER_DISPLAY),
+                          ('10', FILTER_REMOVE),
+                          ],
+                         [(c.label, c.link_type) for c in choices])
+
+    def test_datetime_filter_bridge_from_multi_to_single(self):
+        """
+        Tests that bridge_choices will bridge from range (multi) choices to
+        single choices.
+        """
+        # Two birthdays in Jan 2011
+        Person.objects.create(name="Joe", date_of_birth=date(2011, 1, 10))
+        Person.objects.create(name="Peter", date_of_birth=date(2011, 1, 20))
+
+        # Chosen year = 2010 - 2011
+        params = MultiValueDict({'date_of_birth':['2010..2011']})
+
+        f = DateTimeFilter('date_of_birth', Person, params)
+        qs = Person.objects.all()
+        qs_filtered = f.apply_filter(qs)
+        choices = f.get_choices(qs_filtered)
+
+        # Expect 2010 - 2011 as remove link
+        self.assertEqual(['2010-2011'], [c.label for c in choices if c.link_type == FILTER_REMOVE])
+        # Expect 2011 and January as display
+        self.assertEqual(['2011', 'January'], [c.label for c in choices if c.link_type == FILTER_DISPLAY])
+        # Expect '10' and '20' as choices
+        self.assertEqual(['10', '20'], [c.label for c in choices if c.link_type == FILTER_ADD])
+
+    def test_datetime_filter_day_ranges_end(self):
+        """
+        Test that the ranges for day selection end at the right point (e.g. 31)
+        """
+        # September
+        for i in range(1, 30):
+            Person.objects.create(name="Joe", date_of_birth=date(2011, 9, i))
+
+        params = MultiValueDict({'date_of_birth':['2011-09']})
+
+        f = DateTimeFilter('date_of_birth', Person, params)
+        qs = Person.objects.all()
+        qs_filtered = f.apply_filter(qs)
+        choices = f.get_choices(qs_filtered)
+        self.assertEqual(choices[-1].label[-3:], "-30")
+
+        # October
+        for i in range(2, 31):
+            Person.objects.create(name="Joe", date_of_birth=date(2011, 10, i))
+
+        params = MultiValueDict({'date_of_birth':['2011-10']})
+
+        f = DateTimeFilter('date_of_birth', Person, params)
+        qs = Person.objects.all()
+        qs_filtered = f.apply_filter(qs)
+        choices = f.get_choices(qs_filtered)
+        self.assertEqual(choices[-1].label[-3:], "-31")
+
     def test_numericrange_filter_simple_vals(self):
         # If data is less than max_links, we should get a simple list of values.
         filter1 = NumericRangeFilter('price', Book, MultiValueDict(), max_links=20)
@@ -811,3 +927,33 @@ class TestFilters(TestCase):
         # Should be same after sorting by 'label' (that is equal to Genre.name,
         # and Genre ordering is by that field)
         self.assertEqual(choices2, sorted(choices2, key=operator.attrgetter('label')))
+
+
+class TestCustomFilters(TestCase):
+
+    fixtures = ['django_easyfilters_tests']
+
+    def test_render_choice_object(self):
+        for field, filter_class, test_str in [
+            ('genre', ForeignKeyFilter, u"~~Fantasy"),
+            ('authors', ManyToManyFilter, u"~~Charles"),
+            ('binding', ValuesFilter, u"~~H~~"),
+            ('binding', ChoicesFilter, u"~~Hardback~~"),
+            ('price', NumericRangeFilter, re.compile(u"\~\~\d+")),
+            ('date_published', DateTimeFilter, re.compile('\~\~\d{4}')),
+            ]:
+
+            class CustomFilter(filter_class):
+                def render_choice_object(self, obj):
+                    return u"~~%s~~" % super(CustomFilter, self).render_choice_object(obj)
+
+            class BookFilterSet(FilterSet):
+                fields = [
+                    (field, {}, CustomFilter)
+                    ]
+
+            fs = BookFilterSet(Book.objects.all(), QueryDict(''))
+            if hasattr(test_str, 'search'):
+                self.assertTrue(test_str.search(fs.render()), "%s does not allow customization via render_choice_object" % filter_class)
+            else:
+                self.assertTrue(test_str in fs.render(), "%s does not allow customization via render_choice_object" % filter_class)
