@@ -34,7 +34,13 @@ class Book(models.Model):
     price = models.DecimalField(max_digits=6, decimal_places=2)
     date_published = models.DateField()
     edition = models.IntegerField(default=1)
+    rating = models.FloatField(null=True)
 
     def __unicode__(self):
         return self.name
 
+
+class Person(models.Model):
+    date_of_birth = models.DateField()
+    name = models.CharField(max_length=50)
+
